#!/usr/bin/env python3

# Copyright © 2020-2025, Meheret Tesfaye Batu <meherett.batu@gmail.com>
# Distributed under the MIT software license, see the accompanying
# file COPYING or https://opensource.org/license/mit

from ..slip44 import CoinTypes
from ..ecc import SLIP10Secp256k1ECC
from ..const import (
    Info, Entropies, Mnemonics, Seeds, HDs, Addresses, AddressTypes, Networks, Params, XPrivateKeyVersions, XPublicKeyVersions
)
from .icryptocurrency import (
    ICryptocurrency, INetwork
)


class Mainnet(INetwork):

    TYPE = 0x00
    XPRIVATE_KEY_VERSIONS = XPrivateKeyVersions({
        "P2PKH": 0x488ade4
    })
    XPUBLIC_KEY_VERSIONS = XPublicKeyVersions({
        "P2PKH": 0x488b21e
    })
    WIF_PREFIX = 0x80


class Testnet(INetwork):

    TYPE = 0x10
    XPRIVATE_KEY_VERSIONS = XPrivateKeyVersions({
        "P2PKH": 0x04358394
    })
    XPUBLIC_KEY_VERSIONS = XPublicKeyVersions({
        "P2PKH": 0x043587cf
    })
    WIF_PREFIX = 0xef


class Ergo(ICryptocurrency):

    NAME = "Ergo"
    SYMBOL = "ERG"
    INFO = Info({
        "SOURCE_CODE": "https://github.com/ergoplatform/ergo",
        "WHITEPAPER": "https://ergoplatform.org/en/documents",
        "WEBSITES": [
            "https://ergoplatform.org"
        ]
    })
    ECC = SLIP10Secp256k1ECC
    COIN_TYPE = CoinTypes.Ergo
    NETWORKS = Networks({
        "MAINNET": Mainnet, "TESTNET": Testnet
    })
    DEFAULT_NETWORK = NETWORKS.MAINNET
    ENTROPIES = Entropies({
        "BIP39"
    })
    MNEMONICS = Mnemonics({
        "BIP39"
    })
    SEEDS = Seeds({
        "BIP39"
    })
    HDS = HDs({
        "BIP32", "BIP44"
    })
    DEFAULT_HD = HDS.BIP44
    ADDRESSES = Addresses({
        "ERGO": "Ergo"
    })
    DEFAULT_ADDRESS = ADDRESSES.ERGO
    ADDRESS_TYPES = AddressTypes({
        "P2PKH": "p2pkh",
        "P2SH": "p2sh"
    })
    DEFAULT_ADDRESS_TYPE = ADDRESS_TYPES.P2PKH
    PARAMS = Params({
        "CHECKSUM_LENGTH": 4,
        "ADDRESS_TYPES": {
            "P2PKH": 0x01,  # Pay to Public Key Hash
            "P2SH": 0x02  # Pay to Script Hash
        }
    })



#!/usr/bin/env python3

# Copyright © 2020-2025, Meheret Tesfaye Batu <meherett.batu@gmail.com>
# Distributed under the MIT software license, see the accompanying
# file COPYING or https://opensource.org/license/mit

from ..slip44 import CoinTypes
from ..ecc import SLIP10Secp256k1ECC
from ..const import (
    Info, Entropies, Mnemonics, Seeds, HDs, Addresses, Networks, XPrivateKeyVersions, XPublicKeyVersions
)
from .icryptocurrency import (
    ICryptocurrency, INetwork
)


class Mainnet(INetwork):

    HRP = "iaa"
    XPRIVATE_KEY_VERSIONS = XPrivateKeyVersions({
        "P2PKH": 0x488ade4
    })
    XPUBLIC_KEY_VERSIONS = XPublicKeyVersions({
        "P2PKH": 0x488b21e
    })
    WIF_PREFIX = 0x80


class IRISnet(ICryptocurrency):

    NAME = "IRISnet"
    SYMBOL = "IRIS"
    INFO = Info({
        "SOURCE_CODE": "https://github.com/irisnet",
        "WHITEPAPER": "https://www.irisnet.org/docs/resources/whitepaper-en.html",
        "WEBSITES": [
            "https://www.irisnet.org"
        ]
    })
    ECC = SLIP10Secp256k1ECC
    COIN_TYPE = CoinTypes.IRISnet
    NETWORKS = Networks({
        "MAINNET": Mainnet
    })
    DEFAULT_NETWORK = NETWORKS.MAINNET
    ENTROPIES = Entropies({
        "BIP39"
    })
    MNEMONICS = Mnemonics({
        "BIP39"
    })
    SEEDS = Seeds({
        "BIP39"
    })
    HDS = HDs({
        "BIP32", "BIP44"
    })
    DEFAULT_HD = HDS.BIP44
    ADDRESSES = Addresses({
        "COSMOS": "Cosmos"
    })
    DEFAULT_ADDRESS = ADDRESSES.COSMOS



#!/usr/bin/env python3

# Copyright © 2020-2025, Meheret Tesfaye Batu <meherett.batu@gmail.com>
# Distributed under the MIT software license, see the accompanying
# file COPYING or https://opensource.org/license/mit

from ..slip44 import CoinTypes
from ..ecc import SLIP10Secp256k1ECC
from ..const import (
    Info, Entropies, Mnemonics, Seeds, HDs, Addresses, AddressTypes, Networks, Params, XPrivateKeyVersions, XPublicKeyVersions
)
from .icryptocurrency import (
    ICryptocurrency, INetwork
)


class Mainnet(INetwork):

    HRP = "avax"
    XPRIVATE_KEY_VERSIONS = XPrivateKeyVersions({
        "P2PKH": 0x488ade4
    })
    XPUBLIC_KEY_VERSIONS = XPublicKeyVersions({
        "P2PKH": 0x488b21e
    })
    WIF_PREFIX = 0x80


class Avalanche(ICryptocurrency):

    NAME = "Avalanche"
    SYMBOL = "AVAX"
    INFO = Info({
        "SOURCE_CODE": "https://github.com/ava-labs/avalanchego",
        "WHITEPAPER": "https://www.avalabs.org/whitepapers",
        "WEBSITES": [
            "https://avax.network",
            "https://www.avalabs.org"
        ]
    })
    ECC = SLIP10Secp256k1ECC
    COIN_TYPE = CoinTypes.Avalanche
    NETWORKS = Networks({
        "MAINNET": Mainnet
    })
    DEFAULT_NETWORK = NETWORKS.MAINNET
    ENTROPIES = Entropies({
        "BIP39"
    })
    MNEMONICS = Mnemonics({
        "BIP39"
    })
    SEEDS = Seeds({
        "BIP39"
    })
    HDS = HDs({
        "BIP32", "BIP44"
    })
    DEFAULT_HD = HDS.BIP44
    ADDRESSES = Addresses({
        "AVALANCHE": "Avalanche", "ETHEREUM": "Ethereum"
    })
    DEFAULT_ADDRESS = ADDRESSES.AVALANCHE
    ADDRESS_TYPES = AddressTypes({
        "C_CHAIN": "c-chain",
        "P_CHAIN": "p-chain",
        "X_CHAIN": "x-chain"
    })
    DEFAULT_ADDRESS_TYPE = ADDRESS_TYPES.P_CHAIN
    PARAMS = Params({
        "ADDRESS_TYPES": {
            "P_CHAIN": "P-",  # The Platform Chain (P-Chain) prefix
            "X_CHAIN": "X-"  # The Exchange Chain (X-Chain) prefix
        }
    })



#!/usr/bin/env python3

# Copyright © 2020-2024, Meheret Tesfaye Batu <meherett.batu@gmail.com>
# Distributed under the MIT software license, see the accompanying
# file COPYING or https://opensource.org/license/mit

from typing import (
    Optional, Union, Type
)

from ..ecc import IEllipticCurveCryptography
from ..derivations.bip44 import BIP44Derivation
from ..const import (
    Info, WitnessVersions, Entropies, Mnemonics, Seeds, HDs, Addresses, AddressTypes, AddressPrefixes, Networks, Params, XPrivateKeyVersions, XPublicKeyVersions
)
from ..exceptions import NetworkError


class INetwork:

    # Bitcoin network types
    PUBLIC_KEY_ADDRESS_PREFIX: Optional[int] = None
    SCRIPT_ADDRESS_PREFIX: Optional[int] = None
    HRP: Optional[str] = None
    WITNESS_VERSIONS: Optional[WitnessVersions] = None
    XPRIVATE_KEY_VERSIONS: Optional[XPrivateKeyVersions] = None
    XPUBLIC_KEY_VERSIONS: Optional[XPublicKeyVersions] = None
    MESSAGE_PREFIX: Optional[str] = None
    WIF_PREFIX: Optional[int] = None

    # Bitcoin-Cash, Bitcoin-Cash-SLP and eCash network types
    LEGACY_PUBLIC_KEY_ADDRESS_PREFIX: Optional[int] = None
    STD_PUBLIC_KEY_ADDRESS_PREFIX: Optional[int] = None
    LEGACY_SCRIPT_ADDRESS_PREFIX: Optional[int] = None
    STD_SCRIPT_ADDRESS_PREFIX: Optional[int] = None

    # Monero network types
    STANDARD: Optional[int] = None
    INTEGRATED: Optional[int] = None
    SUB_ADDRESS: Optional[int] = None

    # Cardano network types
    TYPE: Optional[int] = None
    PAYMENT_ADDRESS_HRP: Optional[str] = None
    REWARD_ADDRESS_HRP: Optional[str] = None

    @classmethod
    def name(cls) -> str:
        return cls.__name__.lower()


class ICryptocurrency:

    NAME: str
    SYMBOL: str
    INFO: Info
    ECC: Type[IEllipticCurveCryptography]
    COIN_TYPE: int
    SUPPORT_BIP38: bool = False
    NETWORKS: Networks
    DEFAULT_NETWORK: INetwork
    ENTROPIES: Entropies
    MNEMONICS: Mnemonics
    SEEDS: Seeds
    HDS: HDs
    DEFAULT_HD: str
    ADDRESSES: Addresses
    DEFAULT_ADDRESS: str
    ADDRESS_TYPES: Optional[AddressTypes] = None
    DEFAULT_ADDRESS_TYPE: Optional[str] = None
    ADDRESS_PREFIXES: Optional[AddressPrefixes] = None
    DEFAULT_ADDRESS_PREFIX: Optional[str] = None
    PARAMS: Optional[Params] = None
    DEFAULT_SEMANTIC: str = "p2pkh"

    @classmethod
    def get_default_path(cls, network: Union[str, Type[INetwork]]) -> str:
        try:
            if not isinstance(network, str) and issubclass(network, INetwork):
                network = network.__name__.lower()
            if not cls.NETWORKS.is_network(network=network):
                raise NetworkError(
                    f"Wrong {cls.NAME} network", expected=cls.NETWORKS.networks(), got=network
                )

            bip44_derivation: BIP44Derivation = BIP44Derivation(
                account=0, change="external-chain", address=0
            )
            bip44_derivation.from_coin_type(
                coin_type=cls.COIN_TYPE if network == "mainnet" else 1
            )
            return bip44_derivation.path()
        except TypeError:
            raise NetworkError(
                "Invalid network type", expected=[str, INetwork], got=type(network)
            )



#!/usr/bin/env python3

# Copyright © 2020-2025, Meheret Tesfaye Batu <meherett.batu@gmail.com>
# Distributed under the MIT software license, see the accompanying
# file COPYING or https://opensource.org/license/mit

from typing import (
    Tuple, Union, Optional, Dict, Type
)

from ..ecc import (
    ECCS as EllipticCurveCryptographies, IEllipticCurveCryptography
)
from ..utils import (
    normalize_index, normalize_derivation, index_tuple_to_string
)
from ..exceptions import DerivationError
from .iderivation import IDerivation


class ECCS:

    SLIP10_Secp256k1: str = "SLIP10-Secp256k1"
    SLIP10_Ed25519: str = "SLIP10-Ed25519"
    SLIP10_Nist256p1: str = "SLIP10-Nist256p1"
    KHOLAW_ED25519: str = "Kholaw-Ed25519"
    SLIP10_Ed25519_Blake2b: str = "SLIP10-Ed25519-Blake2b"
    SLIP10_Ed25519_Monero: str = "SLIP10-Ed25519-Monero"


class HDWDerivation(IDerivation):
    """
    This class implements the HDW standard for hierarchical deterministic wallets.
    HDW defines a specific path structure for deriving keys from a master seed.

    .. note::
        This class inherits from the ``IDerivation`` class, thereby ensuring that all functions are accessible.

    +------------------------+------------------------+
    | Name                   | Value                  |
    +========================+========================+
    | SLIP10_Secp256k1       | SLIP10-Secp256k1       |
    +------------------------+------------------------+
    | SLIP10_Ed25519         | SLIP10-Ed25519         |
    +------------------------+------------------------+
    | SLIP10_Nist256p1       | SLIP10-Nist256p1       |
    +------------------------+------------------------+
    | KHOLAW_ED25519         | Kholaw-Ed25519         |
    +------------------------+------------------------+ 
    | SLIP10_Ed25519_Blake2b | SLIP10-Ed25519-Blake2b |
    +------------------------+------------------------+
    | SLIP10_Ed25519_Monero  | SLIP10-Ed25519-Monero  |
    +------------------------+------------------------+
    """

    _account: Union[Tuple[int, bool], Tuple[int, int, bool]]
    _ecc: Tuple[int, bool]
    _address: Union[Tuple[int, bool], Tuple[int, int, bool]]
    eccs: Dict[str, int] = {
        "SLIP10-Secp256k1": 0,
        "SLIP10-Ed25519": 1,
        "SLIP10-Nist256p1": 2,
        "Kholaw-Ed25519": 3,
        "SLIP10-Ed25519-Blake2b": 4,
        "SLIP10-Ed25519-Monero": 5
    }

    def __init__(
        self,
        account: Union[str, int, Tuple[int, int]] = 0,
        ecc: Optional[Union[str, int, Type[IEllipticCurveCryptography]]] = "SLIP10-Secp256k1",
        address: Union[str, int, Tuple[int, int]] = 0
    ) -> None:
        """
        Initialize a HDW derivation path with specified parameters.

        :param account: The HDW account index or tuple. Defaults to 0.
        :type account: Union[str, int, Tuple[int, int]]
        :param ecc: The HDW ecc index. 
        :type ecc: Union[str, int, Type[IEllipticCurveCryptography]]
        :param address: The HDW address index or tuple. Defaults to 0.
        :type address: Union[str, int, Tuple[int, int]]

        :return: None
        """
        super(HDWDerivation, self).__init__()

        self.excepted_ecc = [
            *self.eccs.keys(),
            *self.eccs.values(),
            *EllipticCurveCryptographies.classes(),
            *map(str, self.eccs.values())
        ]
        if ecc not in self.excepted_ecc:
            raise DerivationError(
                f"Bad {self.name()} ecc index", expected=self.excepted_ecc, got=ecc
            )
        ecc = ecc if type(ecc) in [str, int] else ecc.NAME

        self._account = normalize_index(index=account, hardened=True)
        self._ecc = normalize_index(
            index=(self.eccs[ecc] if ecc in self.eccs.keys() else ecc), hardened=False
        )
        self._address = normalize_index(index=address, hardened=False)
        self._path, self._indexes, self._derivations = normalize_derivation(path=(
            f"m/"
            f"{index_tuple_to_string(index=self._account)}/"
            f"{index_tuple_to_string(index=self._ecc)}/"
            f"{index_tuple_to_string(index=self._address)}"
        ))

    @classmethod
    def name(cls) -> str:
        """
        Get the name of the derivation class.

        :return: The name of the derivation class.
        :rtype: str
        """

        return "HDW"


    def from_account(self, account: Union[str, int, Tuple[int, int]]) -> "HDWDerivation":
        """
        Set the object's `_account` attribute to the specified account index or tuple,
        updating `_path`, `_indexes`, and `_derivations` accordingly.

        :param account: The account index or tuple to set. Can be a string, integer, or tuple of two integers.
        :type account: Union[str, int, Tuple[int, int]]

        :return: The updated `HDWDerivation` object itself after setting the account.
        :rtype: HDWDerivation
        """

        self._account = normalize_index(index=account, hardened=True)
        self._path, self._indexes, self._derivations = normalize_derivation(path=(
            f"m/"
            f"{index_tuple_to_string(index=self._account)}/"
            f"{index_tuple_to_string(index=self._ecc)}/"
            f"{index_tuple_to_string(index=self._address)}"
        ))
        return self

    def from_ecc(self, ecc: Union[str, int, Type[IEllipticCurveCryptography]]) -> "HDWDerivation":
        """
        Set the object's `_ecc` attribute to the specified ecc index or key,
        updating `_path`, `_indexes`, and `_derivations` accordingly.

        :param ecc: The ecc index or key to set. Can be a string, integer, or one of the predefined keys.
        :type ecc: Union[str, int, Type[IEllipticCurveCryptography]]

        :return: The updated `HDWDerivation` object itself after setting the ecc.
        :rtype: HDWDerivation
        """

        if ecc not in self.excepted_ecc:
            raise DerivationError(
                f"Bad {self.name()} ecc index", expected=self.excepted_ecc, got=ecc
            )
        ecc = ecc if type(ecc) in [str, int] else ecc.NAME
        self._ecc = normalize_index(
            index=(self.eccs[ecc] if ecc in self.eccs.keys() else ecc), hardened=False
        )
        self._path, self._indexes, self._derivations = normalize_derivation(path=(
            f"m/"
            f"{index_tuple_to_string(index=self._account)}/"
            f"{index_tuple_to_string(index=self._ecc)}/"
            f"{index_tuple_to_string(index=self._address)}"
        ))
        return self

    def from_address(self, address: Union[str, int, Tuple[int, int]]) -> "HDWDerivation":
        """
        Set the object's `_address` attribute to the specified address index or tuple of indexes,
        updating `_path`, `_indexes`, and `_derivations` accordingly.

        :param address: The address index or tuple of indexes to set. Should be non-hardened.
        :type address: Union[str, int, Tuple[int, int]]

        :return: The updated `HDWDerivation` object itself after setting the address.
        :rtype: HDWDerivation
        """

        self._address = normalize_index(index=address, hardened=False)
        self._path, self._indexes, self._derivations = normalize_derivation(path=(
            f"m/"
            f"{index_tuple_to_string(index=self._account)}/"
            f"{index_tuple_to_string(index=self._ecc)}/"
            f"{index_tuple_to_string(index=self._address)}"
        ))
        return self

    def clean(self) -> "HDWDerivation":
        """
        Reset the object's attributes related to HDW derivation to their initial states or defaults.

        :return: The updated `HDWDerivation` object itself after cleaning.
        :rtype: HDWDerivation
        """

        self._account = normalize_index(index=0, hardened=True)
        self._address = normalize_index(index=0, hardened=False)
        self._path, self._indexes, self._derivations = normalize_derivation(path=(
            f"m/"
            f"{index_tuple_to_string(index=self._account)}/"
            f"{index_tuple_to_string(index=self._ecc)}/"
            f"{index_tuple_to_string(index=self._address)}"
        ))
        return self

    def account(self) -> int:
        """
        Retrieve the account value from the object's `_account` attribute.

        Checks the length of `_account`. If it equals 3, returns the second
        element; otherwise, returns the first element.

        :return: The account value stored in `_account`.
        :rtype: int
        """

        return (
            self._account[1] if len(self._account) == 3 else self._account[0]
        )

    def ecc(self) -> str:
        """
        Retrieve the ecc value from the object's eccs dictionary.

        Iterates through the `eccs` dictionary, and if a value matches the first element of `_ecc`,
        sets the corresponding key as the ecc value.

        :return: The key from the `eccs` dictionary that corresponds to the `_ecc` value, or `None` if not found.
        :rtype: str
        """

        _ecc: Optional[str] = None
        for key, value in self.eccs.items():
            if value == self._ecc[0]:
                _ecc = key
                break
        return _ecc

    def address(self) -> int:
        """
        Retrieve the address from the object.

        :return: The address value.
        :rtype: int
        """

        return (
            self._address[1] if len(self._address) == 3 else self._address[0]
        )



#!/usr/bin/env python3

# Copyright © 2020-2025, Meheret Tesfaye Batu <meherett.batu@gmail.com>
# Distributed under the MIT software license, see the accompanying
# file COPYING or https://opensource.org/license/mit

from ..slip44 import CoinTypes
from ..ecc import SLIP10Secp256k1ECC
from ..const import (
    Info, WitnessVersions, Entropies, Mnemonics, Seeds, HDs, Addresses, Networks, Params, XPrivateKeyVersions, XPublicKeyVersions
)
from .icryptocurrency import (
    ICryptocurrency, INetwork
)


class Mainnet(INetwork):

    PUBLIC_KEY_ADDRESS_PREFIX = 0x00
    SCRIPT_ADDRESS_PREFIX = 0x05
    HRP = "bc"
    WITNESS_VERSIONS = WitnessVersions({
        "P2TR": 0x01,
        "P2WPKH": 0x00,
        "P2WSH": 0x00
    })
    XPRIVATE_KEY_VERSIONS = XPrivateKeyVersions({
        "P2PKH": 0x0488ade4,
        "P2SH": 0x0488ade4,
        "P2TR": 0x0488ade4,
        "P2WPKH": 0x04b2430c,
        "P2WPKH_IN_P2SH": 0x049d7878,
        "P2WSH": 0x02aa7a99,
        "P2WSH_IN_P2SH": 0x0295b005
    })
    XPUBLIC_KEY_VERSIONS = XPublicKeyVersions({
        "P2PKH": 0x0488b21e,
        "P2SH": 0x0488b21e,
        "P2TR": 0x0488b21e,
        "P2WPKH": 0x04b24746,
        "P2WPKH_IN_P2SH": 0x049d7cb2,
        "P2WSH": 0x02aa7ed3,
        "P2WSH_IN_P2SH": 0x0295b43f
    })
    MESSAGE_PREFIX = "\x18Bitcoin Signed Message:\n"
    WIF_PREFIX = 0x80


class Testnet(INetwork):

    PUBLIC_KEY_ADDRESS_PREFIX = 0x6f
    SCRIPT_ADDRESS_PREFIX = 0xc4
    HRP = "tb"
    WITNESS_VERSIONS = WitnessVersions({
        "P2TR": 0x01,
        "P2WPKH": 0x00,
        "P2WSH": 0x00
    })
    XPRIVATE_KEY_VERSIONS = XPrivateKeyVersions({
        "P2PKH": 0x04358394,
        "P2SH": 0x04358394,
        "P2TR": 0x04358394,
        "P2WPKH": 0x045f18bc,
        "P2WPKH_IN_P2SH": 0x044a4e28,
        "P2WSH": 0x02575048,
        "P2WSH_IN_P2SH": 0x024285b5
    })
    XPUBLIC_KEY_VERSIONS = XPublicKeyVersions({
        "P2PKH": 0x043587cf,
        "P2SH": 0x043587cf,
        "P2TR": 0x043587cf,
        "P2WPKH": 0x045f1cf6,
        "P2WPKH_IN_P2SH": 0x044a5262,
        "P2WSH": 0x02575483,
        "P2WSH_IN_P2SH": 0x024289ef
    })
    MESSAGE_PREFIX = "\x18Bitcoin Signed Message:\n"
    WIF_PREFIX = 0xef


class Regtest(Testnet):

    HRP = "bcrt"


class Bitcoin(ICryptocurrency):

    NAME = "Bitcoin"
    SYMBOL = "BTC"
    INFO = Info({
        "SOURCE_CODE": "https://github.com/bitcoin/bitcoin",
        "WHITEPAPER": "https://bitcoin.org/bitcoin.pdf",
        "WEBSITES": [
            "https://bitcoin.org"
        ]
    })
    ECC = SLIP10Secp256k1ECC
    COIN_TYPE = CoinTypes.Bitcoin
    SUPPORT_BIP38 = True
    NETWORKS = Networks({
        "MAINNET": Mainnet, "TESTNET": Testnet, "REGTEST": Regtest
    })
    DEFAULT_NETWORK = NETWORKS.MAINNET
    ENTROPIES = Entropies((
        "BIP39", {"ELECTRUM_V1": "Electrum-V1"}, {"ELECTRUM_V2": "Electrum-V2"}
    ))
    MNEMONICS = Mnemonics((
        "BIP39", {"ELECTRUM_V1": "Electrum-V1"}, {"ELECTRUM_V2": "Electrum-V2"}
    ))
    SEEDS = Seeds((
        "BIP39", {"ELECTRUM_V1": "Electrum-V1"}, {"ELECTRUM_V2": "Electrum-V2"}
    ))
    HDS = HDs((
        "BIP32", "BIP44", "BIP49", "BIP84", "BIP86", "BIP141", {"ELECTRUM_V1": "Electrum-V1"}, {"ELECTRUM_V2": "Electrum-V2"}
    ))
    DEFAULT_HD = HDS.BIP44
    ADDRESSES = Addresses((
        "P2PKH", "P2SH", "P2TR", "P2WPKH", {"P2WPKH_IN_P2SH": "P2WPKH-In-P2SH"}, "P2WSH", {"P2WSH_IN_P2SH": "P2WSH-In-P2SH"}
    ))
    DEFAULT_ADDRESS = ADDRESSES.P2PKH
    PARAMS = Params({
        "ALPHABET": "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz",
        "FIELD_SIZE": 0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFC2F,
        "TAP_TWEAK_SHA256": "e80fe1639c9ca050e3af1b39c143c63e429cbceb15d940fbb5c5a1f4af57c5e9"
    })



#!/usr/bin/env python3

# Copyright © 2020-2025, Meheret Tesfaye Batu <meherett.batu@gmail.com>
# Distributed under the MIT software license, see the accompanying
# file COPYING or https://opensource.org/license/mit

from ..slip44 import CoinTypes
from ..ecc import SLIP10Ed25519ECC
from ..const import (
    Info, Entropies, Mnemonics, Seeds, HDs, Addresses, AddressPrefixes, Networks, Params, XPrivateKeyVersions, XPublicKeyVersions
)
from .icryptocurrency import (
    ICryptocurrency, INetwork
)


class Mainnet(INetwork):

    XPRIVATE_KEY_VERSIONS = XPrivateKeyVersions({
        "P2PKH": 0x0488ade4
    })
    XPUBLIC_KEY_VERSIONS = XPublicKeyVersions({
        "P2PKH": 0x0488b21e
    })


class Tezos(ICryptocurrency):

    NAME = "Tezos"
    SYMBOL = "XTZ"
    INFO = Info({
        "SOURCE_CODE": "https://github.com/tezos/tezos",
        "WHITEPAPER": "https://tezos.com/whitepaper.pdf",
        "WEBSITES": [
            "https://www.tezos.com"
        ]
    })
    ECC = SLIP10Ed25519ECC
    COIN_TYPE = CoinTypes.Tezos
    NETWORKS = Networks({
        "MAINNET": Mainnet
    })
    DEFAULT_NETWORK = NETWORKS.MAINNET
    ENTROPIES = Entropies({
        "BIP39"
    })
    MNEMONICS = Mnemonics({
        "BIP39"
    })
    SEEDS = Seeds({
        "BIP39"
    })
    HDS = HDs({
        "BIP32", "BIP44"
    })
    DEFAULT_HD = HDS.BIP44
    ADDRESSES = Addresses({
        "TEZOS": "Tezos"
    })
    DEFAULT_ADDRESS = ADDRESSES.TEZOS
    ADDRESS_PREFIXES = AddressPrefixes({
        "TZ1": "tz1",
        "TZ2": "tz2",
        "TZ3": "tz3"
    })
    DEFAULT_ADDRESS_PREFIX = ADDRESS_PREFIXES.TZ1
    PARAMS = Params({
        "ADDRESS_PREFIXES": {
            "TZ1": b"\x06\xa1\x9f",
            "TZ2": b"\x06\xa1\xa1",
            "TZ3": b"\x06\xa1\xa4"
        }
    })



#!/usr/bin/env python3

# Copyright © 2020-2024, Meheret Tesfaye Batu <meherett.batu@gmail.com>
# Distributed under the MIT software license, see the accompanying
# file COPYING or https://opensource.org/license/mit

from abc import (
    ABC, abstractmethod
)
from typing import (
    Union, Optional
)

from ..mnemonics import IMnemonic


class ISeed(ABC):

    _name: str
    _seed: str

    def __init__(self, seed: str, **kwargs) -> None:
        """
        Initialize an object with a seed value.

        :param seed: The seed value used for initialization.
        :type seed: str

        :return: No return
        :rtype: NoneType
        """

        self._seed = seed

    @classmethod
    def name(cls) -> str:
        pass

    def seed(self) -> str:
        """
        Retrieves the seed associated with the current instance.

        :return: The seed as a string.
        :rtype: str
        """

        return self._seed

    @classmethod
    @abstractmethod
    def from_mnemonic(cls, mnemonic: Union[str, IMnemonic]) -> str:
        pass



#!/usr/bin/env python3

# Copyright © 2020-2025, Meheret Tesfaye Batu <meherett.batu@gmail.com>
# Distributed under the MIT software license, see the accompanying
# file COPYING or https://opensource.org/license/mit

from ..slip44 import CoinTypes
from ..ecc import SLIP10Ed25519ECC
from ..const import (
    Info, Entropies, Mnemonics, Seeds, HDs, Addresses, AddressTypes, Networks, Params, XPrivateKeyVersions, XPublicKeyVersions
)
from .icryptocurrency import (
    ICryptocurrency, INetwork
)


class Mainnet(INetwork):

    XPRIVATE_KEY_VERSIONS = XPrivateKeyVersions({
        "P2PKH": 0x0488ade4
    })
    XPUBLIC_KEY_VERSIONS = XPublicKeyVersions({
        "P2PKH": 0x0488b21e
    })


class PiNetwork(ICryptocurrency):

    NAME = "Pi-Network"
    SYMBOL = "PI"
    INFO = Info({
        "SOURCE_CODE": "https://github.com/pi-apps",
        "WHITEPAPER": "https://minepi.com/white-paper",
        "WEBSITES": [
            "https://minepi.com"
        ]
    })
    ECC = SLIP10Ed25519ECC
    COIN_TYPE = CoinTypes.PiNetwork
    NETWORKS = Networks({
        "MAINNET": Mainnet
    })
    DEFAULT_NETWORK = NETWORKS.MAINNET
    ENTROPIES = Entropies({
        "BIP39"
    })
    MNEMONICS = Mnemonics({
        "BIP39"
    })
    SEEDS = Seeds({
        "BIP39"
    })
    HDS = HDs({
        "BIP32", "BIP44"
    })
    DEFAULT_HD = HDS.BIP44
    ADDRESSES = Addresses({
        "STELLAR": "Stellar"
    })
    DEFAULT_ADDRESS = ADDRESSES.STELLAR
    ADDRESS_TYPES = AddressTypes({
        "PRIVATE_KEY": "private_key",
        "PUBLIC_KEY": "public_key"
    })
    DEFAULT_ADDRESS_TYPE = ADDRESS_TYPES.PUBLIC_KEY
    PARAMS = Params({
        "CHECKSUM_LENGTH": 2,
        "ADDRESS_TYPES": {
            "PRIVATE_KEY": 18 << 3,
            "PUBLIC_KEY": 6 << 3
        }
    })



#!/usr/bin/env python3

# Copyright © 2020-2024, Meheret Tesfaye Batu <meherett.batu@gmail.com>
# Distributed under the MIT software license, see the accompanying
# file COPYING or https://opensource.org/license/mit

from typing import List

from .mnemonic import (
    MoneroMnemonic, MONERO_MNEMONIC_WORDS, MONERO_MNEMONIC_LANGUAGES
)


__all__: List[str] = [
    "MoneroMnemonic",
    "MONERO_MNEMONIC_WORDS",
    "MONERO_MNEMONIC_LANGUAGES"
]
